SystemOrganization addCategory: #'Zinc-Resource-Meta-Core'!

Object subclass: #ZnMimeType
	instanceVariableNames: 'main sub parameters'
	classVariableNames: 'ExtensionsMap SubTypeBinaryCache'
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnMimeType commentStamp: '<historical>' prior: 0!
I am ZnMimeType.
Mime types are an element used in describing resources, more specifically their format.

Here are some examples MIME types:

	text/plain
	text/html
	text/plain;charset=utf-8
	image/png
	text/*
 
A mime type has a main/sub form with optional parameters. 

For more information: http://en.wikipedia.org/wiki/MIME_type
I know whether I am binary or not and know my charset when applicable.
My class side offers access to some common constants.
Note that for text types we default to UTF-8 encoding.

Examples:

	ZnMimeType textPlain = 'text/plain;charset=utf-8' asZnMimeType.
	ZnMimeType textPlain charSet.
	ZnMimeType textHtml matches: ZnMimeType text.
	ZnMimeType default matches: ZnMimeType any.
	ZnMimeType imagePng isBinary.
	ZnMimeType forFilenameExtension: 'html'.

I started life as a copy of WAMineType.

Part of Zinc HTTP Components.!

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:46'!
any
	"This wildcard instance #matches: any mime type"
	
	^ self main: '*' sub: '*'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:34'!
applicationFormUrlEncoded
	^ self main: 'application' sub: 'x-www-form-urlencoded'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:35'!
applicationJavascript
	^ self main: 'application' sub: 'javascript'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:35'!
applicationJson
	^ self main: 'application' sub: 'json'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:35'!
applicationOctetStream
	^ self main: 'application' sub: 'octet-stream'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:35'!
applicationPdf
	^ self main: 'application' sub: 'pdf'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/11/2014 15:09'!
applicationSton
	^ self main: 'application' sub: 'ston'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:36'!
applicationXml
	^ self main: 'application' sub: 'xml'! !

!ZnMimeType class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/15/2013 15:36'!
commonMimeTypes
	"Return a list of commom mime types, the ones we provided convenience accessors for."

	^ #( 
			applicationFormUrlEncoded applicationJavascript applicationJson 
			applicationOctetStream applicationPdf applicationXml 
			imageGif imageJpeg imagePng imageSvg 
			textCss textHtml textJavascript textPlain 
		) collect: [ :each | self perform: each ]! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/16/2010 15:48'!
default
	^ self applicationOctetStream ! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/15/2013 15:48'!
forFilenameExtension: extension
	^ self 
		forFilenameExtension: extension 
		ifAbsent: [ self default ]! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/15/2013 15:48'!
forFilenameExtension: extension ifAbsent: block
	^ ExtensionsMap 
		at: extension 
		ifAbsent: block! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/23/2015 14:39'!
fromString: aString 
	| main endOfSub sub parts parameters |
	main := aString copyUpTo: $/.
	endOfSub := aString indexOf: $;.
	endOfSub := endOfSub = 0 
		ifTrue: [ aString size ]
		ifFalse: [ endOfSub - 1 ].
	sub := aString copyFrom: main size + 2 to: endOfSub.
	endOfSub = aString size ifTrue: [ ^ self main: main sub: sub ].
	parts := (aString copyFrom: endOfSub + 1 to: aString size) substrings: ';'.
	parameters := Dictionary new.
	parts do: [ :each | 
		parameters
			at: (each copyUpTo: $=) trimBoth 
			put: (each copyAfter: $=) trimBoth ].
	^ self 
		main: main
		sub: sub
		parameters: parameters! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:36'!
imageGif
	^ self main: 'image' sub: 'gif'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:36'!
imageJpeg
	^ self main: 'image' sub: 'jpeg'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:36'!
imagePng
	^ self main: 'image' sub: 'png'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:36'!
imageSvg
	^ self main: 'image' sub: 'svg+xml'! !

!ZnMimeType class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 12/15/2014 16:39'!
initialize
	self 
		initializeExtensionsMap;
		initializeSubTypeBinaryCache! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/15/2013 15:30'!
initializeExtensionsMap
	"Create a mapping between filename extensions and mime types"

	ExtensionsMap := Dictionary new.
	self mimeTypeFilenameExtensionsSpec
		do: [ :each | 
			| mimeType |
			mimeType := self fromString: each first.
			each allButFirstDo: [ :extension | ExtensionsMap at: extension put: mimeType ] ].
	self patchExtensionsMap! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/15/2014 16:44'!
initializeSubTypeBinaryCache
	"This private cache holds sub types of mime types that were classified #isBinary.
	In the future use a limited LRUCache when we no longer have to support Pharo 2.0"
	
	SubTypeBinaryCache := Dictionary new! !

!ZnMimeType class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/15/2013 15:37'!
knownFilenameExtensions
	"Return a collection of the filename extensions that we know of"
	
	^ ExtensionsMap keys! !

!ZnMimeType class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/15/2013 15:38'!
knownMimeTypesWithFilenameExtensions
	"Return a collection of mime types for which we know a filename extension"
	
	^ ExtensionsMap values! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/28/2015 20:19'!
main: aMainString sub: aSubstring
	^ self new
		main: aMainString;
		sub: aSubstring;
		yourself! !

!ZnMimeType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/28/2015 20:19'!
main: aMainString sub: aSubstring parameters: aDictionary
	^ (self main: aMainString sub: aSubstring)
		parameters: aDictionary;
		yourself! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/15/2013 15:10'!
mimeTypeFilenameExtensionsSpec
	"This array maps mime types to a list of filename extensions, based on the Debian/Ubuntu /etc/mime.types file"
	
"('mime.types' asFileReference readStreamDo: [ :in | in contentsOfEntireFile ]) lines 
	collect: [ :line |
		(line isEmpty not and: [ line first ~= $# ])
			ifTrue: [
				(line findTokens: { Character space. Character tab }) asArray ] ]
	thenSelect: [ :each | each notNil and: [ each size > 1 ] ]."
	
	^ #(#('application/andrew-inset' 'ez') #('application/annodex' 'anx') #('application/atom+xml' 'atom') #('application/atomcat+xml' 'atomcat') #('application/atomserv+xml' 'atomsrv') #('application/bbolin' 'lin') #('application/cu-seeme' 'cu') #('application/davmount+xml' 'davmount') #('application/dicom' 'dcm') #('application/dsptype' 'tsp') #('application/ecmascript' 'es') #('application/futuresplash' 'spl') #('application/hta' 'hta') #('application/java-archive' 'jar') #('application/java-serialized-object' 'ser') #('application/java-vm' 'class') #('application/javascript' 'js') #('application/json' 'json') #('application/m3g' 'm3g') #('application/mac-binhex40' 'hqx') #('application/mac-compactpro' 'cpt') #('application/mathematica' 'nb' 'nbp') #('application/mbox' 'mbox') #('application/msaccess' 'mdb') #('application/msword' 'doc' 'dot') #('application/mxf' 'mxf') #('application/octet-stream' 'bin') #('application/oda' 'oda') #('application/ogg' 'ogx') #('application/onenote' 'one' 'onetoc2' 'onetmp' 'onepkg') #('application/pdf' 'pdf') #('application/pgp-encrypted' 'pgp') #('application/pgp-keys' 'key') #('application/pgp-signature' 'sig') #('application/pics-rules' 'prf') #('application/postscript' 'ps' 'ai' 'eps' 'epsi' 'epsf' 'eps2' 'eps3') #('application/rar' 'rar') #('application/rdf+xml' 'rdf') #('application/rtf' 'rtf') #('application/sla' 'stl') #('application/smil' 'smi' 'smil') #('application/xhtml+xml' 'xhtml' 'xht') #('application/xml' 'xml' 'xsl' 'xsd') #('application/xspf+xml' 'xspf') #('application/zip' 'zip') #('application/vnd.android.package-archive' 'apk') #('application/vnd.cinderella' 'cdy') #('application/vnd.google-earth.kml+xml' 'kml') #('application/vnd.google-earth.kmz' 'kmz') #('application/vnd.mozilla.xul+xml' 'xul') #('application/vnd.ms-excel' 'xls' 'xlb' 'xlt') #('application/vnd.ms-excel.addin.macroEnabled.12' 'xlam') #('application/vnd.ms-excel.sheet.binary.macroEnabled.12' 'xlsb') #('application/vnd.ms-excel.sheet.macroEnabled.12' 'xlsm') #('application/vnd.ms-excel.template.macroEnabled.12' 'xltm') #('application/vnd.ms-fontobject' 'eot') #('application/vnd.ms-officetheme' 'thmx') #('application/vnd.ms-pki.seccat' 'cat') #('application/vnd.ms-powerpoint' 'ppt' 'pps') #('application/vnd.ms-powerpoint.addin.macroEnabled.12' 'ppam') #('application/vnd.ms-powerpoint.presentation.macroEnabled.12' 'pptm') #('application/vnd.ms-powerpoint.slide.macroEnabled.12' 'sldm') #('application/vnd.ms-powerpoint.slideshow.macroEnabled.12' 'ppsm') #('application/vnd.ms-powerpoint.template.macroEnabled.12' 'potm') #('application/vnd.ms-word.document.macroEnabled.12' 'docm') #('application/vnd.ms-word.template.macroEnabled.12' 'dotm') #('application/vnd.oasis.opendocument.chart' 'odc') #('application/vnd.oasis.opendocument.database' 'odb') #('application/vnd.oasis.opendocument.formula' 'odf') #('application/vnd.oasis.opendocument.graphics' 'odg') #('application/vnd.oasis.opendocument.graphics-template' 'otg') #('application/vnd.oasis.opendocument.image' 'odi') #('application/vnd.oasis.opendocument.presentation' 'odp') #('application/vnd.oasis.opendocument.presentation-template' 'otp') #('application/vnd.oasis.opendocument.spreadsheet' 'ods') #('application/vnd.oasis.opendocument.spreadsheet-template' 'ots') #('application/vnd.oasis.opendocument.text' 'odt') #('application/vnd.oasis.opendocument.text-master' 'odm') #('application/vnd.oasis.opendocument.text-template' 'ott') #('application/vnd.oasis.opendocument.text-web' 'oth') #('application/vnd.openxmlformats-officedocument.presentationml.presentation' 'pptx') #('application/vnd.openxmlformats-officedocument.presentationml.slide' 'sldx') #('application/vnd.openxmlformats-officedocument.presentationml.slideshow' 'ppsx') #('application/vnd.openxmlformats-officedocument.presentationml.template' 'potx') #('application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' 'xlsx') #('application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' 'xlsx') #('application/vnd.openxmlformats-officedocument.spreadsheetml.template' 'xltx') #('application/vnd.openxmlformats-officedocument.spreadsheetml.template' 'xltx') #('application/vnd.openxmlformats-officedocument.wordprocessingml.document' 'docx') #('application/vnd.openxmlformats-officedocument.wordprocessingml.template' 'dotx') #('application/vnd.rim.cod' 'cod') #('application/vnd.smaf' 'mmf') #('application/vnd.stardivision.calc' 'sdc') #('application/vnd.stardivision.chart' 'sds') #('application/vnd.stardivision.draw' 'sda') #('application/vnd.stardivision.impress' 'sdd') #('application/vnd.stardivision.math' 'sdf') #('application/vnd.stardivision.writer' 'sdw') #('application/vnd.stardivision.writer-global' 'sgl') #('application/vnd.sun.xml.calc' 'sxc') #('application/vnd.sun.xml.calc.template' 'stc') #('application/vnd.sun.xml.draw' 'sxd') #('application/vnd.sun.xml.draw.template' 'std') #('application/vnd.sun.xml.impress' 'sxi') #('application/vnd.sun.xml.impress.template' 'sti') #('application/vnd.sun.xml.math' 'sxm') #('application/vnd.sun.xml.writer' 'sxw') #('application/vnd.sun.xml.writer.global' 'sxg') #('application/vnd.sun.xml.writer.template' 'stw') #('application/vnd.symbian.install' 'sis') #('application/vnd.tcpdump.pcap' 'cap' 'pcap') #('application/vnd.visio' 'vsd') #('application/vnd.wap.wbxml' 'wbxml') #('application/vnd.wap.wmlc' 'wmlc') #('application/vnd.wap.wmlscriptc' 'wmlsc') #('application/vnd.wordperfect' 'wpd') #('application/vnd.wordperfect5.1' 'wp5') #('application/x-123' 'wk') #('application/x-7z-compressed' '7z') #('application/x-abiword' 'abw') #('application/x-apple-diskimage' 'dmg') #('application/x-bcpio' 'bcpio') #('application/x-bittorrent' 'torrent') #('application/x-cab' 'cab') #('application/x-cbr' 'cbr') #('application/x-cbz' 'cbz') #('application/x-cdf' 'cdf' 'cda') #('application/x-cdlink' 'vcd') #('application/x-chess-pgn' 'pgn') #('application/x-comsol' 'mph') #('application/x-cpio' 'cpio') #('application/x-csh' 'csh') #('application/x-debian-package' 'deb' 'udeb') #('application/x-director' 'dcr' 'dir' 'dxr') #('application/x-dms' 'dms') #('application/x-doom' 'wad') #('application/x-dvi' 'dvi') #('application/x-font' 'pfa' 'pfb' 'gsf' 'pcf' 'pcf.Z') #('application/x-font-woff' 'woff') #('application/x-freemind' 'mm') #('application/x-futuresplash' 'spl') #('application/x-ganttproject' 'gan') #('application/x-gnumeric' 'gnumeric') #('application/x-go-sgf' 'sgf') #('application/x-graphing-calculator' 'gcf') #('application/x-gtar' 'gtar') #('application/x-gtar-compressed' 'tgz' 'taz') #('application/x-hdf' 'hdf') #('application/x-hwp' 'hwp') #('application/x-ica' 'ica') #('application/x-info' 'info') #('application/x-internet-signup' 'ins' 'isp') #('application/x-iphone' 'iii') #('application/x-iso9660-image' 'iso') #('application/x-jam' 'jam') #('application/x-java-jnlp-file' 'jnlp') #('application/x-jmol' 'jmz') #('application/x-kchart' 'chrt') #('application/x-killustrator' 'kil') #('application/x-koan' 'skp' 'skd' 'skt' 'skm') #('application/x-kpresenter' 'kpr' 'kpt') #('application/x-kspread' 'ksp') #('application/x-kword' 'kwd' 'kwt') #('application/x-latex' 'latex') #('application/x-lha' 'lha') #('application/x-lyx' 'lyx') #('application/x-lzh' 'lzh') #('application/x-lzx' 'lzx') #('application/x-maker' 'frm' 'maker' 'frame' 'fm' 'fb' 'book' 'fbdoc') #('application/x-md5' 'md5') #('application/x-mif' 'mif') #('application/x-mpegURL' 'm3u8') #('application/x-ms-wmd' 'wmd') #('application/x-ms-wmz' 'wmz') #('application/x-msdos-program' 'com' 'exe' 'bat' 'dll') #('application/x-msi' 'msi') #('application/x-netcdf' 'nc') #('application/x-ns-proxy-autoconfig' 'pac' 'dat') #('application/x-nwc' 'nwc') #('application/x-object' 'o') #('application/x-oz-application' 'oza') #('application/x-pkcs7-certreqresp' 'p7r') #('application/x-pkcs7-crl' 'crl') #('application/x-python-code' 'pyc' 'pyo') #('application/x-qgis' 'qgs' 'shp' 'shx') #('application/x-quicktimeplayer' 'qtl') #('application/x-rdp' 'rdp') #('application/x-redhat-package-manager' 'rpm') #('application/x-rss+xml' 'rss') #('application/x-ruby' 'rb') #('application/x-scilab' 'sci' 'sce') #('application/x-scilab-xcos' 'xcos') #('application/x-sh' 'sh') #('application/x-sha1' 'sha1') #('application/x-shar' 'shar') #('application/x-shockwave-flash' 'swf' 'swfl') #('application/x-silverlight' 'scr') #('application/x-sql' 'sql') #('application/x-stuffit' 'sit' 'sitx') #('application/x-sv4cpio' 'sv4cpio') #('application/x-sv4crc' 'sv4crc') #('application/x-tar' 'tar') #('application/x-tcl' 'tcl') #('application/x-tex-gf' 'gf') #('application/x-tex-pk' 'pk') #('application/x-texinfo' 'texinfo' 'texi') #('application/x-trash' '~' '%' 'bak' 'old' 'sik') #('application/x-troff' 't' 'tr' 'roff') #('application/x-troff-man' 'man') #('application/x-troff-me' 'me') #('application/x-troff-ms' 'ms') #('application/x-ustar' 'ustar') #('application/x-wais-source' 'src') #('application/x-wingz' 'wz') #('application/x-x509-ca-cert' 'crt') #('application/x-xcf' 'xcf') #('application/x-xfig' 'fig') #('application/x-xpinstall' 'xpi') #('audio/amr' 'amr') #('audio/amr-wb' 'awb') #('audio/amr' 'amr') #('audio/amr-wb' 'awb') #('audio/annodex' 'axa') #('audio/basic' 'au' 'snd') #('audio/csound' 'csd' 'orc' 'sco') #('audio/flac' 'flac') #('audio/midi' 'mid' 'midi' 'kar') #('audio/mpeg' 'mpga' 'mpega' 'mp2' 'mp3' 'm4a') #('audio/mpegurl' 'm3u') #('audio/ogg' 'oga' 'ogg' 'spx') #('audio/prs.sid' 'sid') #('audio/x-aiff' 'aif' 'aiff' 'aifc') #('audio/x-gsm' 'gsm') #('audio/x-mpegurl' 'm3u') #('audio/x-ms-wma' 'wma') #('audio/x-ms-wax' 'wax') #('audio/x-pn-realaudio' 'ra' 'rm' 'ram') #('audio/x-realaudio' 'ra') #('audio/x-scpls' 'pls') #('audio/x-sd2' 'sd2') #('audio/x-wav' 'wav') #('chemical/x-alchemy' 'alc') #('chemical/x-cache' 'cac' 'cache') #('chemical/x-cache-csf' 'csf') #('chemical/x-cactvs-binary' 'cbin' 'cascii' 'ctab') #('chemical/x-cdx' 'cdx') #('chemical/x-cerius' 'cer') #('chemical/x-chem3d' 'c3d') #('chemical/x-chemdraw' 'chm') #('chemical/x-cif' 'cif') #('chemical/x-cmdf' 'cmdf') #('chemical/x-cml' 'cml') #('chemical/x-compass' 'cpa') #('chemical/x-crossfire' 'bsd') #('chemical/x-csml' 'csml' 'csm') #('chemical/x-ctx' 'ctx') #('chemical/x-cxf' 'cxf' 'cef') #('chemical/x-embl-dl-nucleotide' 'emb' 'embl') #('chemical/x-galactic-spc' 'spc') #('chemical/x-gamess-input' 'inp' 'gam' 'gamin') #('chemical/x-gaussian-checkpoint' 'fch' 'fchk') #('chemical/x-gaussian-cube' 'cub') #('chemical/x-gaussian-input' 'gau' 'gjc' 'gjf') #('chemical/x-gaussian-log' 'gal') #('chemical/x-gcg8-sequence' 'gcg') #('chemical/x-genbank' 'gen') #('chemical/x-hin' 'hin') #('chemical/x-isostar' 'istr' 'ist') #('chemical/x-jcamp-dx' 'jdx' 'dx') #('chemical/x-kinemage' 'kin') #('chemical/x-macmolecule' 'mcm') #('chemical/x-macromodel-input' 'mmd' 'mmod') #('chemical/x-mdl-molfile' 'mol') #('chemical/x-mdl-rdfile' 'rd') #('chemical/x-mdl-rxnfile' 'rxn') #('chemical/x-mdl-sdfile' 'sd' 'sdf') #('chemical/x-mdl-tgf' 'tgf') #('chemical/x-mmcif' 'mcif') #('chemical/x-mol2' 'mol2') #('chemical/x-molconn-Z' 'b') #('chemical/x-mopac-graph' 'gpt') #('chemical/x-mopac-input' 'mop' 'mopcrt' 'mpc' 'zmt') #('chemical/x-mopac-out' 'moo') #('chemical/x-mopac-vib' 'mvb') #('chemical/x-ncbi-asn1' 'asn') #('chemical/x-ncbi-asn1-ascii' 'prt' 'ent') #('chemical/x-ncbi-asn1-binary' 'val' 'aso') #('chemical/x-ncbi-asn1-spec' 'asn') #('chemical/x-pdb' 'pdb' 'ent') #('chemical/x-rosdal' 'ros') #('chemical/x-swissprot' 'sw') #('chemical/x-vamas-iso14976' 'vms') #('chemical/x-vmd' 'vmd') #('chemical/x-xtel' 'xtel') #('chemical/x-xyz' 'xyz') #('image/gif' 'gif') #('image/ief' 'ief') #('image/jpeg' 'jpeg' 'jpg' 'jpe') #('image/pcx' 'pcx') #('image/png' 'png') #('image/svg+xml' 'svg' 'svgz') #('image/tiff' 'tiff' 'tif') #('image/vnd.djvu' 'djvu' 'djv') #('image/vnd.microsoft.icon' 'ico') #('image/vnd.wap.wbmp' 'wbmp') #('image/x-canon-cr2' 'cr2') #('image/x-canon-crw' 'crw') #('image/x-cmu-raster' 'ras') #('image/x-coreldraw' 'cdr') #('image/x-coreldrawpattern' 'pat') #('image/x-coreldrawtemplate' 'cdt') #('image/x-corelphotopaint' 'cpt') #('image/x-epson-erf' 'erf') #('image/x-jg' 'art') #('image/x-jng' 'jng') #('image/x-ms-bmp' 'bmp') #('image/x-nikon-nef' 'nef') #('image/x-olympus-orf' 'orf') #('image/x-photoshop' 'psd') #('image/x-portable-anymap' 'pnm') #('image/x-portable-bitmap' 'pbm') #('image/x-portable-graymap' 'pgm') #('image/x-portable-pixmap' 'ppm') #('image/x-rgb' 'rgb') #('image/x-xbitmap' 'xbm') #('image/x-xpixmap' 'xpm') #('image/x-xwindowdump' 'xwd') #('message/rfc822' 'eml') #('model/iges' 'igs' 'iges') #('model/mesh' 'msh' 'mesh' 'silo') #('model/vrml' 'wrl' 'vrml') #('model/x3d+vrml' 'x3dv') #('model/x3d+xml' 'x3d') #('model/x3d+binary' 'x3db') #('text/cache-manifest' 'appcache') #('text/calendar' 'ics' 'icz') #('text/css' 'css') #('text/csv' 'csv') #('text/h323' '323') #('text/html' 'html' 'htm' 'shtml') #('text/iuls' 'uls') #('text/mathml' 'mml') #('text/plain' 'asc' 'txt' 'text' 'pot' 'brf' 'srt') #('text/richtext' 'rtx') #('text/scriptlet' 'sct' 'wsc') #('text/texmacs' 'tm') #('text/tab-separated-values' 'tsv') #('text/vnd.sun.j2me.app-descriptor' 'jad') #('text/vnd.wap.wml' 'wml') #('text/vnd.wap.wmlscript' 'wmls') #('text/x-bibtex' 'bib') #('text/x-boo' 'boo') #('text/x-c++hdr' 'h++' 'hpp' 'hxx' 'hh') #('text/x-c++src' 'c++' 'cpp' 'cxx' 'cc') #('text/x-chdr' 'h') #('text/x-component' 'htc') #('text/x-csh' 'csh') #('text/x-csrc' 'c') #('text/x-dsrc' 'd') #('text/x-diff' 'diff' 'patch') #('text/x-haskell' 'hs') #('text/x-java' 'java') #('text/x-lilypond' 'ly') #('text/x-literate-haskell' 'lhs') #('text/x-moc' 'moc') #('text/x-pascal' 'p' 'pas') #('text/x-pcs-gcd' 'gcd') #('text/x-perl' 'pl' 'pm') #('text/x-python' 'py') #('text/x-scala' 'scala') #('text/x-setext' 'etx') #('text/x-sfv' 'sfv') #('text/x-sh' 'sh') #('text/x-tcl' 'tcl' 'tk') #('text/x-tex' 'tex' 'ltx' 'sty' 'cls') #('text/x-vcalendar' 'vcs') #('text/x-vcard' 'vcf') #('video/3gpp' '3gp') #('video/annodex' 'axv') #('video/dl' 'dl') #('video/dv' 'dif' 'dv') #('video/fli' 'fli') #('video/gl' 'gl') #('video/mpeg' 'mpeg' 'mpg' 'mpe') #('video/MP2T' 'ts') #('video/mp4' 'mp4') #('video/quicktime' 'qt' 'mov') #('video/ogg' 'ogv') #('video/webm' 'webm') #('video/vnd.mpegurl' 'mxu') #('video/x-flv' 'flv') #('video/x-la-asf' 'lsf' 'lsx') #('video/x-mng' 'mng') #('video/x-ms-asf' 'asf' 'asx') #('video/x-ms-wm' 'wm') #('video/x-ms-wmv' 'wmv') #('video/x-ms-wmx' 'wmx') #('video/x-ms-wvx' 'wvx') #('video/x-msvideo' 'avi') #('video/x-sgi-movie' 'movie') #('video/x-matroska' 'mpv' 'mkv') #('x-conference/x-cooltalk' 'ice') #('x-epoc/x-sisx-app' 'sisx') #('x-world/x-vrml' 'vrm' 'vrml' 'wrl'))! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:37'!
multiPartFormData
	^ self main: 'multipart' sub: 'form-data'! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/15/2013 15:39'!
patchExtensionsMap
	"Patch the extension map that we read from the spec"
	
	#('~' '%') do: [ :each | ExtensionsMap removeKey: each ].
	(ExtensionsMap at: 'text') setCharSetUTF8.
	(ExtensionsMap at: 'html') setCharSetUTF8.
	ExtensionsMap at: 'asc' put: 'text/plain' asZnMimeType! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:47'!
text
	"This wildcard instance #matches: all textual mime types"
	
	^ self main: 'text' sub: '*'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:37'!
textCss
	^ self main: 'text' sub: 'css'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:47'!
textHtml
	"Note that the charset is set to utf8 automatically"
	
	^ (self main: 'text' sub: 'html') 
		setCharSetUTF8; 
		yourself! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:38'!
textJavascript
	^ self main: 'text' sub: 'javascript'! !

!ZnMimeType class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 10/15/2013 14:47'!
textPlain
	"Note that the charset is set to utf8 automatically"

	^ (self main: 'text' sub: 'plain') 
		setCharSetUTF8; 
		yourself! !

!ZnMimeType class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/15/2014 16:29'!
textSubTypeFragments
	"Return a collection of fragments of sub types that indicate for any main type
	that the mime type is non-binary (thus textual)"
	
	^ #('javascript' 'json' 'ston' 'html' 'xml' 'sgml' 'yaml') ! !

!ZnMimeType methodsFor: 'comparing' stamp: 'monty 10/14/2015 08:25'!
= other
	"#matches should be used instead of #= to ignore parameters."

	^ (self class == other class)
		and: [ self main = other main
			and: [ self sub = other sub 
				and: [ self hasSameParametersAs: other ] ] ]! !

!ZnMimeType methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 11/15/2013 21:10'!
asMIMEType
	^ self! !

!ZnMimeType methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ self! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
charSet
	^ self parameterAt: 'charset' ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:24'!
charSet: aString
	self parameters at: 'charset' put: aString! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:54'!
clearCharSet
	self removeParameter: 'charset'! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/21/2013 20:24'!
hasParameters
	^ parameters notNil and: [ parameters isEmpty not ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'monty 10/14/2015 08:26'!
hasSameParametersAs: aMimeType
	^ self hasParameters = aMimeType hasParameters
		and: [self parameters size = aMimeType parameters size
			and: [self parameters associations allSatisfy: [:each |
				aMimeType parameters includesAssociation: each]]]! !

!ZnMimeType methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/21/2013 20:32'!
hash
	| hash |
	hash := self main hash bitXor: self sub hash.
	self hasParameters 
		ifTrue: [ hash := hash bitXor: self parameters hash ].
	^ hash! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/23/2015 14:39'!
isBinary
	"Answers whether the contents of a resource described by me 
	should be considered as binary instead of textual."

	self main = 'text' ifTrue: [ ^ false ].
	^ SubTypeBinaryCache
		at: self sub
		ifAbsentPut: [ 
			| subTypeFragments |
			subTypeFragments := self sub substrings: '+-'.
			subTypeFragments noneSatisfy: [ :each | 
				self class textSubTypeFragments includes: each ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/2/2010 11:19'!
isCharSetUTF8
	^ self charSet notNil and: [ self charSet sameAs: 'utf-8' ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/10/2013 11:08'!
isNonStandard
	"Tests if the receiver is a non-standard mime type,
	i.e. one that is not registered with IANA"
	
	^ (self main beginsWith: 'x-')
		or: [ (self main beginsWith: 'X-')
		or: [ (self sub beginsWith: 'x-')
		or: [ self sub beginsWith: 'X-' ] ] ]! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/10/2013 11:08'!
isVendorSpecific
	"Tests if the receiver is a vendor specific mime type"
	
	^ self sub beginsWith: 'vnd.'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
main
	^ main! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 10:51'!
main: aString
	main := aString asLowercase! !

!ZnMimeType methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/7/2013 11:53'!
matches: aMimeType
	"Compare the receiver to aMimeType, which is the pattern to match, 
	it is a normal instance where main or sub can be wildcards.
	Matches should be used instead of #= to ignore parameters."
	
	^ (aMimeType main = '*'
		or: [ aMimeType main = self main ])
		and: [
			aMimeType sub = '*'
				or: [ aMimeType sub = self sub ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 11:00'!
parameterAt: key
	^ self parameterAt: key ifAbsent: [ KeyNotFound signalFor: key ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
parameterAt: key ifAbsent: block
	parameters isNil ifTrue: [ ^ block value ].
	^ self parameters at: key ifAbsent: block! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 9/7/2010 14:50'!
parameterAt: aKey put: aValue
	self parameters at: aKey put: aValue! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2011 20:40'!
parameters
	parameters isNil ifTrue: [ parameters := SmallDictionary new ].
	^ parameters! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
parameters: aDictionary
	parameters := aDictionary! !

!ZnMimeType methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 12/6/2011 20:41'!
postCopy
	super postCopy.
	parameters := parameters copy! !

!ZnMimeType methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:49'!
printOn: stream
	stream
		nextPutAll: self main;
		nextPut: $/;
		nextPutAll: self sub.
	parameters isNil ifFalse: [ 
		parameters keysAndValuesDo: [ :key :value | 
			stream
				nextPut: $;;
				nextPutAll: key;
				nextPut: $=;
				nextPutAll: value ] ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 3/6/2012 10:53'!
removeParameter: key
	parameters isNil ifTrue: [ ^ self ].
	^ self parameters removeKey: key ifAbsent: [ nil ]! !

!ZnMimeType methodsFor: 'parameters' stamp: 'SvenVanCaekenberghe 12/2/2010 11:20'!
setCharSetUTF8
	self charSet: 'utf-8'! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 14:47'!
sub
	^ sub! !

!ZnMimeType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 10:51'!
sub: aString
	sub := aString asLowercase! !

Object subclass: #ZnResourceMetaUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnResourceMetaUtils commentStamp: '<historical>' prior: 0!
I am ZnResourceMetaUtils.

I contain some class side utility methods.

Part of Zinc HTTP Components.!

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/14/2014 15:07'!
decodePercent: string
	^ ZnPercentEncoder new 
		decodePlusAsSpace: false; 
		decode: string! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/14/2014 15:07'!
decodePercentForQuery: string
	^ ZnPercentEncoder new 
		decodePlusAsSpace: true; 
		decode: string! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'KrisGybels 6/11/2014 11:37'!
encodePercent: string safeSet: safeSet encoder: encoder
	^ ZnPercentEncoder new
		characterEncoder: encoder;
		safeSet: (self perform: safeSet);
		encode: string! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/14/2014 15:07'!
parseQueryFrom: stream
	| string index fields |
	fields := ZnMultiValueDictionary new.
	[ stream atEnd ] whileFalse: [
		string := stream upTo: $&.
		index := string indexOf: $=.
		index > 0
			ifFalse: [ fields at: (self decodePercentForQuery: string) put: nil ]
			ifTrue: [
				fields
					at: (self decodePercentForQuery: (string copyFrom: 1 to: index - 1))
					add: (self decodePercentForQuery: (string copyFrom: index + 1 to: string size)) ] ].
	^ fields! !

!ZnResourceMetaUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/15/2014 10:21'!
queryKeyValueSafeSet
	"When a query is interpreted as a list of key=value&.. pairs,
	it is better to encode = and & and leave them out of the safe set.
	Furthermore, since + is interpreted as space in the query part,
	it is unsafe as well"
	
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~!!$''()*,;:@/?'! !

!ZnResourceMetaUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/24/2013 10:22'!
querySafeSet
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~!!$&''()*+,;=:@/?'! !

!ZnResourceMetaUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/24/2013 09:39'!
rfc3986SafeSet
	^ ZnPercentEncoder rfc3986UnreservedCharacters ! !

!ZnResourceMetaUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/24/2013 10:22'!
urlPathSafeSet
	^ 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789-_.~!!$&''()*+,;=:@'! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'KrisGybels 6/11/2014 13:47'!
writeQueryFields: dictionary on: stream
	self writeQueryFields: dictionary withEncoder: ZnUTF8Encoder new on: stream ! !

!ZnResourceMetaUtils class methodsFor: 'conversions' stamp: 'KrisGybels 6/11/2014 13:40'!
writeQueryFields: dictionary withEncoder: encoder on: stream
	| first |
	first := true.
	dictionary keysAndValuesDo: [ :key :value | 
		first ifFalse: [ stream nextPut: $& ].
		first := false.
		stream nextPutAll: (self encodePercent: key safeSet: #queryKeyValueSafeSet encoder: encoder).
		value ifNotNil: [ 
			stream
				nextPut: $=;
				nextPutAll: (self encodePercent: value safeSet: #queryKeyValueSafeSet encoder: encoder) ] ]! !

Object subclass: #ZnUrl
	instanceVariableNames: 'scheme host port segments query fragment username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnUrl commentStamp: 'SvenVanCaekenberghe 2/5/2015 00:00' prior: 0!
I am ZnUrl, an implementation of an interpreted URL/URI.
URLs are an element used in describing resources, more specifically to identify them.

I consist of the following parts:
  - scheme - like #http, #https, #ws, #wws, #file or nil
  - host - hostname string or nil
  - port - port integer or nil
  - segments - collection of path segments, ends with #/ for directories
  - query - query dictionary or nil
  - fragment - fragment string or nil
  - username - username string or nil
  - password - password string or nil

The syntax of my external representation informally looks like this

  scheme://username:password@host:port/segments?query#fragment

I am most often created by parsing my external representation using either my #fromString: class method or by sending the #asZnUrl convenience method to a String. Using #asZnUrl helps in accepting both Strings and ZnUrls arguments.

  ZnUrl fromString: 'http://www.google.com/search?q=Smalltalk'.

I can also be constucted programmatically.

  ZnUrl new 
    scheme: #https; 
    host: 'encrypted.google.com'; 
    addPathSegment: 'search'; 
    queryAt: 'q' put: 'Smalltalk'; 
    yourself.
  
My components can be manipulated destructively. Here is an example:

  'http://www.google.com/?one=1&two=2' asZnUrl
    queryAt: 'three' put: '3';
    queryRemoveKey: 'one';
    yourself.

Some characters of parts of a URL are illegal because they would interfere with the syntax and further processing and thus have to be encoded. The methods in accessing protocols do not do any encoding, those in parsing and printing do. Here is an example:

  'http://www.google.com' asZnUrl
    addPathSegment: 'some encoding here';
    queryAt: 'and some encoding' put: 'here, too';
    yourself

My parser is somewhat forgiving and accepts some unencoded URLs as well, like most browsers would.

  'http://www.example.com:8888/a path?q=a, b, c' asZnUrl.

I can parse in the context of a default scheme, like a browser would do.

  ZnUrl fromString: 'www.example.com' defaultScheme: #http

Given a scheme, I know its default port, try #portOrDefault.

A path defaults to what is commonly referred to as slash, test with #isSlash. Paths are most often (but don't have to be) interpreted as filesystem paths. To support this, I have #isFilePath and #isDirectoryPath tests and #file and #directory accessors.

I have some support to handle one URL in the context of another one, this is also known as a relative URL in the context of an absolute URL. Refer to #isAbsolute, #isRelative and #inContextOf:

  '/folder/file.txt' asZnUrl inContextOf: 'http://fileserver.example.net:4400' asZnUrl.

Incomplete relative references can be parsed and resolved in the context of a base URL using #withRelativeReference:

  'http://www.site.com/static/html/home.html' asZnUrl withRelativeReference: '../js/menu.js'.

Sometimes, the combination of my host and port are referred to as authority, see #authority.

URL/URI/URN (Uniform/Universal Resource Locator/Identifier/Name) are closely related and can be and are used as synonyms is many contexts. Refer to http://en.wikipedia.org/wiki/Url for more information.

There is a convenience method #retrieveContents to download the resource a ZnUrl points to,

  'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents.

This is implemented using a ZnUrlOperation.
 
Part of Zinc HTTP Components.!

!ZnUrl class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2015 10:47'!
defaultPortForScheme: scheme
	(#(http ws) includes: scheme) ifTrue: [ ^ 80 ].
	(#(https wss) includes: scheme) ifTrue: [ ^ 443 ].
	^ nil! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/3/2011 19:35'!
fromString: string
	^ (self new)
		parseFrom: string; 
		yourself ! !

!ZnUrl class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
fromString: string defaultScheme: defaultScheme
	^ (self new)
		parseFrom: string defaultScheme: defaultScheme; 
		yourself ! !

!ZnUrl class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/8/2016 00:03'!
image
	"Return a File URL refering to the file path of the current image file.
	This is useful as a context to resolve relative URLs, see #withRelativeReference:"
	
	^ SmalltalkImage current imagePath asPath asZnUrl! !

!ZnUrl class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/12/2014 14:20'!
schemesNotUsingDoubleSlash
	"Most URL schemes use a double slash, as in http://
	but some don't, return a list of those"
	
	^ #( #mailto #telnet )! !

!ZnUrl class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/30/2013 14:38'!
schemesNotUsingPath
	"Most URL schemes use a hierarchical path
	but some don't, return a list of those"
	
	^ #( #mailto )! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:27'!
& association
	^ self withQuery: association ! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/19/2015 11:03'!
+ string
	^ self withRelativeReference: string! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/16/2013 20:22'!
/ object
	^ object addedToZnUrl: self! !

!ZnUrl methodsFor: 'comparing' stamp: 'monty 4/23/2015 13:15'!
= anObject
	^ self == anObject
		or: [ self class == anObject class and: [ self equals: anObject ] ]! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:27'!
? association
	^ self withQuery: association ! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 7/9/2012 16:18'!
addPathSegment: segment
	"Modify the receiver's path by adding segment at the end"
	
	segments isNil ifTrue: [ 
		segments := OrderedCollection new ].
	segments add: segment! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 7/9/2012 16:18'!
addPathSegments: pathSegments
	"Modify the receiver's path by adding the elements of pathSegments at the end"

	pathSegments do: [ :each | self addPathSegment: each ]! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/16/2013 20:39'!
addedToZnUrl: url
	^ url withPathSegments: self pathSegments ! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 5/19/2015 11:02'!
asFileUrl
	"Return a copy of the receiver as a File URL, replacing the scheme with #file.
	This is useful when you want to do a #asFileReference afterwards."
	
	^ self copy
		scheme: #file;
		yourself! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/6/2013 19:21'!
asRelativeUrl
	"Copy the receiver, with scheme, host and port cleared"
	
	^ self copy
		scheme: nil;
		host: nil;
		port: nil;
		yourself! !

!ZnUrl methodsFor: 'converting' stamp: 'MarcusDenker 11/14/2013 10:37'!
asUrl
	^ self! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/4/2011 11:24'!
asZnUrl
	^ self! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/11/2011 12:05'!
asZnUrlWithDefaults
	^ self copy 
		setDefaults; 
		yourself! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
authority
	^ self hasNonDefaultPort 
		ifTrue: [ 
			String streamContents: [ :stream | 
				stream nextPutAll: self host; nextPut: $:; print: self port ] ]
		ifFalse: [ 
			self host ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/6/2015 16:23'!
authorityWithPort
	^ String streamContents: [ :stream | 
			stream nextPutAll: self host; nextPut: $:; print: self portOrDefault ] ! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:32'!
clearPath
	self hasPath
		ifTrue: [ segments removeAll ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 4/29/2015 14:47'!
closePath
	self isDirectoryPath
		ifFalse: [ self addPathSegment: #/ ]! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/30/2013 16:27'!
decodePercent: string
	^ ZnResourceMetaUtils decodePercent: string! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:55'!
directory
	^ self isDirectoryPath
		ifTrue: [ self path ]
		ifFalse: [ 
			String streamContents: [ :stream |
				segments allButLast  
					do: [ :each | stream nextPutAll: each ]
					separatedBy: [ stream nextPut: $/ ] ] ]! !

!ZnUrl methodsFor: 'private' stamp: 'KrisGybels 6/11/2014 11:47'!
encode: string on: stream
	stream nextPutAll: (ZnResourceMetaUtils encodePercent: string safeSet: #rfc3986SafeSet encoder: ZnUTF8Encoder new)! !

!ZnUrl methodsFor: 'private' stamp: 'KrisGybels 6/11/2014 11:47'!
encodePath: string on: stream
	stream nextPutAll: (ZnResourceMetaUtils encodePercent: string safeSet: #urlPathSafeSet encoder: ZnUTF8Encoder new)! !

!ZnUrl methodsFor: 'private' stamp: 'KrisGybels 6/11/2014 11:47'!
encodeQuery: string on: stream
	stream nextPutAll: (ZnResourceMetaUtils encodePercent: string safeSet: #querySafeSet encoder: ZnUTF8Encoder new)! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 1/30/2013 17:23'!
enforceKnownScheme
	(scheme isNil or: [ #(#http #https #ws #wss #file) includes: scheme ])
		ifFalse: [ (ZnUnknownScheme scheme: scheme) signal ]! !

!ZnUrl methodsFor: 'comparing' stamp: 'monty 4/23/2015 13:15'!
equals: url
	self scheme = url scheme ifFalse: [ ^ false ].
	self host = url host ifFalse: [ ^ false ].
	self portOrDefault = url portOrDefault ifFalse: [ ^ false ].
	((self isSlash and: [ url isSlash ]) or: [ self segments = url segments ]) ifFalse: [ ^ false ].
	(self hasQuery or: [ url hasQuery ])
		ifTrue: [ self query = url query ifFalse: [ ^ false ] ].
	self fragment = url fragment ifFalse: [ ^ false ].
	^ true! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:39'!
file
	^ self isDirectoryPath
		ifTrue: [ String new ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:46'!
firstPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments first ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment
	^ fragment! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:51'!
fragment: string
	fragment := string! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasFragment
	^ fragment notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasHost
	^ host notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 21:01'!
hasNonDefaultPort
	^ self hasPort 
		and: [ self port ~= (self class defaultPortForScheme: self schemeOrDefault) ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
hasPassword
	^ password notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 12:03'!
hasPath
	^ segments notNil and: [ segments isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasPort
	^ port notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/6/2013 20:12'!
hasQuery
	^ query notNil and: [ query isEmpty not ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:48'!
hasScheme
	^ scheme notNil! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/6/2015 16:25'!
hasSecureScheme
	^ #(https wss) includes: self scheme! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
hasUsername
	^ username notNil! !

!ZnUrl methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:43'!
hash
	^ self hasPath
		ifTrue: [ segments inject: host hash into: [ :sum :each | sum bitXor: each hash ] ]
		ifFalse: [ host hash ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
host
	^ host! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/6/2013 19:19'!
host: hostName
	host := hostName ifNil: [ nil ] ifNotNil: [ hostName asLowercase ]! !

!ZnUrl methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 2/4/2015 23:57'!
inContextOf: absoluteUrl
	"Return a copy of the receiver where scheme, host and port
	are taken from absoluteUrl unless they are already in the receiver.
	Path merging is not supported. See also #withRelativeReference: "
	
	| copy |
	copy := self copy.
	copy hasScheme ifFalse: [ copy scheme: absoluteUrl scheme ].
	copy hasHost ifFalse: [ copy host: absoluteUrl host ].
	(copy hasPort not and: [ copy scheme = absoluteUrl scheme ]) 
		ifTrue: [ copy port: absoluteUrl port ].      
	^ copy! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/5/2015 10:11'!
isAbsolute
	"We consider URLs with scheme://host as absolute (port is not relevant here).
	See also #inContextOf: and #withRelativeReference:"
	
	^ self hasScheme and: [ self hasHost ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:36'!
isDirectoryPath
	^ self hasPath 
		ifTrue: [ segments last = #/ ]
		ifFalse: [ true ]
	! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/30/2013 17:15'!
isEmpty
	^ (self hasScheme | self hasHost | self hasPath | self hasQuery | self hasFragment | self hasUsername | self hasPassword) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:42'!
isFile
	^ scheme = #file! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/14/2013 11:37'!
isFilePath
	^ self isDirectoryPath not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttp
	^ scheme == #http! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:50'!
isHttps
	^ scheme == #https! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/13/2011 10:41'!
isLocalHost
	^ self hasHost and: [ #('localhost' '127.0.0.1') includes: self host ]! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:55'!
isRelative
	^ self isAbsolute not! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/12/2014 14:20'!
isSchemeNotUsingDoubleSlash: schemeString
	^ self class schemesNotUsingDoubleSlash includes: schemeString asLowercase! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/30/2013 14:33'!
isSchemeUsingDoubleSlash
	^ (self class schemesNotUsingDoubleSlash includes: self scheme) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/30/2013 14:38'!
isSchemeUsingPath
	^ (self class schemesNotUsingPath includes: self scheme) not! !

!ZnUrl methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/4/2011 10:54'!
isSlash
	^ self hasPath not or: [ segments size = 1 and: [ segments first = #/ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 1/4/2011 14:51'!
lastPathSegment
	^ self isSlash
		ifTrue: [ nil ]
		ifFalse: [ segments last ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/25/2013 14:48'!
mailToAddress
	"Assuming my scheme is #mailto, return the address."
	
	^ self username, '@', self host ! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 10/24/2013 10:20'!
parseAuthority: string from: start to: stop
	| index |
	((index := string indexOf: $@ startingAt: start) > 0 and: [ index < stop ])
		ifTrue: [
			self parseUserInfo: (ReadStream on: string from: start to: index - 1).
			self parseHostPort: (ReadStream on: string from: index + 1 to: stop) ]
		ifFalse: [ 
			self parseHostPort: (ReadStream on: string from: start to: stop) ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 8/11/2011 14:47'!
parseFrom: string
	self parseFrom: string defaultScheme: nil! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/15/2015 11:42'!
parseFrom: string defaultScheme: defaultScheme
	| start end index |
	start := 1.
	end := string size.
	(index := string indexOf: $#) > 0
		ifTrue: [ 
			self fragment: (self decodePercent: (string copyFrom: index + 1 to: end)).
			end := index - 1 ].
	(index := string indexOf: $?) > 0
		ifTrue: [
			self query: (ZnResourceMetaUtils parseQueryFrom: (ReadStream on: string from: index + 1 to: end)).
			end := index - 1 ].
	((index := string indexOfSubCollection: '://') > 0 and: [ index <= end ])
		ifTrue: [
			self scheme: (string copyFrom: 1 to: index - 1).
			start := index + 3 ]
		ifFalse: [ 
			((index := string indexOf: $:) > 0 
					and: [ index <= end 
						and: [ self isSchemeNotUsingDoubleSlash: (string copyFrom: 1 to: index - 1) ] ])
				ifTrue: [ 
					self scheme: (string copyFrom: 1 to: index - 1).
					start := index + 1 ]
				ifFalse: [ 
					defaultScheme ifNotNil: [ self scheme: defaultScheme ] ] ].
	self hasScheme 
		ifTrue: [  
			(index := string indexOf: $/ startingAt: start) > 0
				ifTrue: [
					self parseAuthority: string from: start to: index - 1.
					start := index ]
				ifFalse: [ 
					^ self parseAuthority: string from: start to: end ] ].
	self parsePath: (ReadStream on: string from: start to: end)! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 10/25/2013 17:00'!
parseHostPort: stream
	| hostString portNumber |
	self isFile
		ifTrue: [ 
			(hostString := stream upTo: $/) isEmpty
				ifFalse: [ self host: (self decodePercent: hostString) ] ]
		ifFalse: [ 
			(hostString := stream upTo: $:) isEmpty
				ifFalse: [ self host: (self decodePercent: hostString) ].
			stream atEnd
				ifFalse: [
					portNumber := Integer readFrom: stream ifFail: [ ZnPortNotANumber signal ].
					(portNumber between: 1 and: 65535) ifFalse: [ DomainError signalFrom: 1 to: 65535 ].
					self port: portNumber ] ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/4/2015 17:04'!
parsePath: stream
	stream peekFor: $/.
	[ stream atEnd ] whileFalse: [ | segment |
		segment := String streamContents: [ :stringStream |
			[ stream atEnd not and: [ stream peek ~= $/ ] ] whileTrue: [
				stringStream nextPut: stream next ] ].
		(segment isEmpty or: [ segment = '.' ])
			ifFalse: [
				segment = '..' 
					ifTrue: [ self removeLastPathSegment ]
					ifFalse: [ 
						segment = '.' 
							ifFalse: [ self addPathSegment: (self decodePercent: segment) ] ] ].
		((stream peekFor: $/) and: [ stream atEnd ])
			ifTrue: [ self closePath ] ]! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:28'!
parseUserInfo: stream
	| userString |
	(userString := stream upTo: $:) isEmpty 
		ifFalse: [  
			self username: (self decodePercent: userString) ].
	stream atEnd 
		ifFalse: [ 
			self password: (self decodePercent: stream upToEnd) ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:42'!
password
	^ password! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
password: string
	password := string! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 19:43'!
path
	self hasPath ifFalse: [ ^ String new ].
	^ String streamContents: [ :stream | 
		segments 
			do: [ :each | 
				each == #/
					ifFalse: [ stream nextPutAll: each ] ]
			separatedBy: [ stream nextPut: $/ ] ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:16'!
pathPrintString
	^ String streamContents: [ :stream | 
			self printPathOn: stream ]! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/4/2011 11:17'!
pathQueryFragmentPrintString
	^ String streamContents: [ :stream | 
			self printPathQueryFragmentOn: stream ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 5/14/2014 11:52'!
pathSegments
	^ segments ifNil: [ #() ]! !

!ZnUrl methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/6/2013 14:13'!
performOperation: operation
	"Look for and execute a handler that can perform operation on the receiver"
	
	^ self performOperation: operation with: nil! !

!ZnUrl methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/6/2013 14:13'!
performOperation: operation with: argument
	"Look for and execute a handler that can perform operation 
	on the receiver with the optional argument"
	
	^ ZnUrlOperation
		performOperation: operation 
		with: argument 
		on: self ! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
port
	^ port! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:14'!
port: integer
	port := integer! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2015 11:41'!
portIfAbsent: valuable
	^ self hasPort
		ifTrue: [ self port ]
		ifFalse: valuable! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2015 11:43'!
portOrDefault
	^ self portIfAbsent: [ self class defaultPortForScheme: self schemeOrDefault ]! !

!ZnUrl methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 4/26/2011 12:34'!
postCopy
	super postCopy.
	segments := segments copy.
	query := query copy! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:21'!
printAuthorityOn: stream
	self hasUsername ifTrue: [
		self encode: self username on: stream.
		self hasPassword ifTrue: [
			stream nextPut: $:.
			self encode: self password on: stream ].
		stream nextPut: $@ ].
	self hasHost ifTrue: [ 
		self encode: self host on: stream ].
	self hasPort ifTrue: [ 
		stream nextPut: $:; print: self port ]
! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/30/2013 14:34'!
printOn: stream
	self hasScheme ifTrue: [ 
		stream nextPutAll: self scheme; nextPut: $:.
		self isSchemeUsingDoubleSlash ifTrue: [ stream nextPut: $/; nextPut: $/ ] ].
	self printAuthorityOn: stream.
	self printPathQueryFragmentOn: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/24/2013 10:08'!
printPathOn: stream
	self hasPath ifFalse: [ ^ stream nextPut: $/ ].
 	segments do: [ :each | 
		stream nextPut: $/.
		each == #/
			ifFalse: [ 
				self encodePath: each on: stream ] ] ! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/30/2013 14:39'!
printPathQueryFragmentOn: stream
	self isSchemeUsingPath 
		ifTrue: [ self printPathOn: stream ].
	self hasQuery
		ifTrue: [ self printQueryOn: stream ].
	self hasFragment
		ifFalse: [ ^ self ].
	stream nextPut: $#.
	self encodeQuery: self fragment on: stream! !

!ZnUrl methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/8/2012 21:02'!
printQueryOn: stream
	stream nextPut: $?. 
	ZnResourceMetaUtils writeQueryFields: self query on: 	stream! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/4/2015 23:50'!
processRelativeReference: reference
	"Parse & resolve the relative reference using myself as base URL, destructively"
	
	| index end |
	reference first = $/ ifTrue: [ self clearPath ].
	(self isFilePath and: [ ('?#' includes: reference first) not ] ) ifTrue: [ self removeLastPathSegment ].
	end := reference size.
	(index := reference indexOf: $#) > 0
		ifTrue: [ 
			self fragment: (self decodePercent: (reference copyFrom: index + 1 to: end)).
			end := index - 1 ].
	(index := reference indexOf: $?) > 0
		ifTrue: [
			self query: (ZnResourceMetaUtils parseQueryFrom: (ReadStream on: reference from: index + 1 to: end)).
			end := index - 1 ].
	self parsePath: (ReadStream on: reference from: 1 to: end).
	(reference = '.' ) | (reference = '..' ) | (reference endsWith: '/.') | (reference endsWith: '/..') 
		ifTrue: [ self closePath ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 16:26'!
query
	^ query! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 20:15'!
query: dictionary
	query := dictionary! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:43'!
queryAddAll: keyedCollection
	"Add all key/value pairs in keyedCollection as query parameters to the receiver.
	Note that we use #addAllMulti:"

	keyedCollection isEmpty
		ifFalse: [ 
			query ifNil: [ query := ZnMultiValueDictionary new ].
			query addAllMulti: keyedCollection ].
	^ keyedCollection! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:44'!
queryAt: key
	"Return the value of the query parameter key in the receiver.
	Signal a KeyNotFound exception if there is no such parameter"
	
	^ self queryAt: key ifAbsent: [ KeyNotFound signalFor: key printString ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:35'!
queryAt: key add: value
	"Modify the receiver by adding a query variable key=value.
	If key already exists, value is added to it."

	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key asString add: (value ifNotNil: [value asString])! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:45'!
queryAt: key ifAbsent: block
	"Return the value of the query parameter key in the receiver.
	Execute block if there is no such parameter"
	
	^ self hasQuery
		ifTrue: [ self query at: key asString ifAbsent: block ]
		ifFalse: block! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:45'!
queryAt: key ifPresent: block
	"Execute block with the value of the query parameter named key as value.
	Return nil if there is no such parameter"
	
	^ self hasQuery
		ifTrue: [ self query at: key asString ifPresent: block ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'KenTreis 01/19/2013 12:35'!
queryAt: key put: value
	"Modify the receiver by setting a query variable key=value.
	If key is already exists, it is overwritten."

	query isNil 
		ifTrue: [ query := ZnMultiValueDictionary new ].
	query at: key asString put: (value ifNotNil: [value asString])! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:46'!
queryDo: block
	"Execute block for each query key/value pair in the receiver"
	
	self hasQuery ifTrue: [ 
		self query keysAndValuesDo: block ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:46'!
queryKeys
	"Return the collection of all query keys in the receiver"
	
	^ self hasQuery
		ifTrue: [ self query keys ]
		ifFalse: [ #() ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:46'!
queryRemoveAll
	"Modify the receiver by removing all query parameters"
	
	self hasQuery
		ifTrue: [ self query removeAll ]! !

!ZnUrl methodsFor: 'accessing-query' stamp: 'SvenVanCaekenberghe 3/26/2014 14:47'!
queryRemoveKey: key
	"Modify the receiver by remove the query parameter named key.
	Do nothing if there is no such parameter"
	
	self hasQuery
		ifTrue: [ self query removeKey: key asString ifAbsent: [ ] ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 18:43'!
removeFirstPathSegment
	self hasPath
		ifTrue: [ segments removeFirst ]! !

!ZnUrl methodsFor: 'accessing-path' stamp: 'SvenVanCaekenberghe 3/16/2013 18:42'!
removeLastPathSegment
	self hasPath
		ifTrue: [ segments removeLast ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/3/2011 15:09'!
scheme
	^ scheme! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/25/2013 15:05'!
scheme: anObject
	anObject isNil
		ifTrue: [ scheme := nil ]
		ifFalse: [ scheme := anObject asLowercase asSymbol ]! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/11/2011 11:05'!
schemeOrDefault
	^ self hasScheme 
		ifTrue: [ self scheme ]
		ifFalse: [ #http ]    ! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 10:56'!
segments
	^ segments! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 13:22'!
segments: collection
	segments := collection! !

!ZnUrl methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/11/2011 12:04'!
setDefaults
	self hasScheme 
		ifFalse: [ self scheme: self schemeOrDefault ].
	self hasPort 
		ifFalse: [ self port: self portOrDefault ]    ! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:42'!
username
	^ username! !

!ZnUrl methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 16:19'!
username: string
	username := string! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:22'!
withPathSegment: segment
	"Return a new URL equal to the receiver with its path extended with segment"
	
	^ self copy
		addPathSegment: segment;
		yourself! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:23'!
withPathSegments: pathSegments
	"Return a new URL equal to the receiver with its path extended with pathSegments"
	
	^ self copy
		addPathSegments: pathSegments;
		yourself! !

!ZnUrl methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/9/2012 16:25'!
withQuery: association
	"Return a new URL equal to the receiver with the association's 
	key=value added as a query parameter.
	Note that #queryAt:add: is used."
	
	^ self copy
		queryAt: association key add: association value;
		yourself 
		! !

!ZnUrl methodsFor: 'parsing' stamp: 'SvenVanCaekenberghe 2/5/2015 22:24'!
withRelativeReference: reference
	"Return a new URL by parsing & resolving the relative reference using myself as base URL.
	This implements the process described in RFC 3986 Section 5"
	
	| resolved |
	[ (resolved := reference asZnUrl) isAbsolute 
		ifTrue: [ ^ resolved ] ] on: Error do: [ ].
	(reference beginsWith: '//')
		ifTrue: [ ^ self class fromString: (reference allButFirst: 2) defaultScheme: self scheme ].
	resolved := self copy.
	resolved fragment: nil.
	reference isEmpty ifTrue: [ ^ resolved ].
	reference first = $# ifFalse: [ resolved queryRemoveAll ].
	resolved processRelativeReference: reference.
	^ resolved! !

Object subclass: #ZnUrlOperation
	instanceVariableNames: 'url argument'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnUrlOperation commentStamp: 'SvenVanCaekenberghe 10/24/2014 15:13' prior: 0!
I am ZnUrlOperation.
I am an abtract superclass of operations that can be performed on ZnUrl objects.

Some operations, like retrieving the contents of a resource described by a URL, depend on elements, most likely the scheme, of the URL. By creating the appropriate subclasses, this behavior can be added.

I implement searching/matching logic on my class side.

  #performOperation:with:on:

which defaults to matching the #operation and #schemes.

I am instanciated to be executed. 

  #performOperation

A concrete example is ZnUrl>>#retrieveContents with subclasses ZnHttpRetrieveContents and ZnFileRetrieveContents.

Part of Zinc HTTP Components.!

!ZnUrlOperation class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 10/24/2014 15:49'!
handlerForOperation: operation with: argument on: url
	"Search for a subclass that can handle operation with argument on url.
	Signal an Error when no matching handler is found."

	self subclassesDo: [ :each |
		(each handlesOperation: operation with: argument on: url)
			ifTrue: [ ^ each ] ].
	self error: 
		('No URL hander found for operation {1} on URL {2} with {3}' 
			format: { operation. url. argument })! !

!ZnUrlOperation class methodsFor: 'acccessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:45'!
handlesOperation: operation with: argument on: url
	"Return true when my instances can handle operation with argument on url.
	This can optionally be overridden for a more specific test."
	
	^ self operation = operation and: [ self schemes includes: url scheme ]! !

!ZnUrlOperation class methodsFor: 'acccessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:26'!
operation
	"Return the operation that I implement.
	Subclasses can/should override this method."
	
	^ nil! !

!ZnUrlOperation class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/6/2013 13:47'!
performOperation: operation with: argument on: url
	"Search for a subclass that can handle operation with argument on url.
	Instanciate the operation, execute it and return the result.
	Signal an Error when no matching handler is found."
	
	| handlerClass |
	handlerClass := self handlerForOperation: operation with: argument on: url.
	^ handlerClass new
		url: url;
		argument: argument;
		performOperation ! !

!ZnUrlOperation class methodsFor: 'acccessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:27'!
schemes
	"Return a collection of scheme identifiers that I handle.
	Subclasses can/should override this method."
	
	^ #( )! !

!ZnUrlOperation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:19'!
argument
	
	^ argument! !

!ZnUrlOperation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:19'!
argument: anObject
	
	argument := anObject! !

!ZnUrlOperation methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/6/2013 13:28'!
performOperation
	"Perform the actual operation using url and the optional argument."
	
	self subclassResponsibility 
! !

!ZnUrlOperation methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/6/2013 14:01'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(; print: self class operation; space; print: self url.
	self argument ifNotNil: [ :arg | stream space; print: arg ].
	stream nextPut: $)! !

!ZnUrlOperation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:19'!
url
	
	^ url! !

!ZnUrlOperation methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 13:27'!
url: aUrl
	
	url := aUrl! !

!Collection methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 3/16/2013 20:24'!
addedToZnUrl: url 
	^ url withPathSegments: self! !

Dictionary subclass: #ZnMultiValueDictionary
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnMultiValueDictionary commentStamp: '<historical>' prior: 0!
I am ZnMultiValueDictionary. 
I am a Dictionary.

I offer an #at:add: method to transparently create Array valued multi entries when needed.
My #keysAndValuesDo: is overwritten to transparently deal with multi entries.
To merge two instance preserving multiple values you can use #addAllMulti:
I normally limit the number of entries to protect me from resource abuse.

Note that most other methods will show the actual value.

Part of Zinc HTTP Components.!

!ZnMultiValueDictionary class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/23/2015 16:45'!
defaultLimit
	"Return the maximum number dictionary entries to accept.
	Used by ZnMultiValueDictionary and thus for reading headers, 
	url query and application form url encoded entity fields.
	This helps to protect us from malicious content."
	
	^ ZnMaximumNumberOfDictionaryEntries value ifNil: [ 256 ]! !

!ZnMultiValueDictionary methodsFor: 'adding' stamp: 'KenTreis 01/19/2013 10:41'!
add: anAssociation
	self checkLimitForKey: anAssociation key.
	^ super add: anAssociation! !

!ZnMultiValueDictionary methodsFor: 'adding' stamp: 'SvenVanCaekenberghe 1/7/2013 10:28'!
addAllMulti: keyedCollection
	"Note that we use #at:add:"

	keyedCollection keysAndValuesDo: [ :key :value | 
		self at: key add: value ]! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:19'!
at: key add: value
	"Store value under key. If key already exists, store multiple values as Array"
	
	| existingValue newValue |
	existingValue := self at: key ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [
			existingValue isArray 
				ifTrue: [ existingValue copyWith: value ]
				ifFalse: [ Array with: existingValue with: value ] ].
	^ self at: key put: newValue 

! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'KenTreis 01/19/2013 10:41'!
at: key put: anObject
	self checkLimitForKey: key.
	^ super at: key put: anObject! !

!ZnMultiValueDictionary methodsFor: 'private' stamp: 'KenTreis 01/19/2013 10:42'!
checkLimitForKey: aKey
	"Signal an exception when the limit, if present, is exceeded."
	
	(self limit notNil and: [ self size >= self limit and: [ (self includesKey: aKey) not ]])
		ifTrue: [ (ZnTooManyDictionaryEntries limit: self limit) signal ].
! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/23/2015 16:39'!
initialize: n
	super initialize: n.
	limit := self class defaultLimit! !

!ZnMultiValueDictionary methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/12/2011 11:10'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	super keysAndValuesDo: [ :key :value |
		value isArray
			ifTrue: [ 
				value do: [ :each | 
					block value: key value: each ] ]
			ifFalse: [
				block value: key value: value ] ]! !

!ZnMultiValueDictionary methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2013 12:38'!
limit
	^ limit! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:35'!
limit: numberOfEntries
	limit := numberOfEntries ! !

!ZnMultiValueDictionary methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 19:36'!
unlimited
	self limit: nil! !

!FileStream methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 10/25/2013 16:48'!
asZnUrl
	"Convert the receiver in a file:// ZnUrl"

	| fileUrl |
	fileUrl := ZnUrl new.
	fileUrl scheme: #file.
	self directory pathSegments do: [ :each | fileUrl addPathSegment: each ].
	fileUrl addPathSegment: self localName.
	^ fileUrl! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 3/16/2013 20:23'!
addedToZnUrl: url
	| segments |
	segments := self findTokens: '/'.
	^ url withPathSegments: segments! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 1/4/2011 19:47'!
asZnMimeType
	^ ZnMimeType fromString: self! !

!String methodsFor: '*zinc-resource-meta-core' stamp: 'SvenVanCaekenberghe 1/4/2011 13:03'!
asZnUrl
	^ ZnUrl fromString: self! !

DynamicVariable subclass: #ZnMaximumNumberOfDictionaryEntries
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnMaximumNumberOfDictionaryEntries commentStamp: 'SvenVanCaekenberghe 4/23/2015 16:38' prior: 0!
I am ZnMaximumNumberOfDictionaryEntries.
I am a DynamicVariable and a ProcessSpecificVariable.

I am used to control the maximum number of entries being added to a limited ZnMultiValueDictionary.
!

Error subclass: #ZnPortNotANumber
	instanceVariableNames: 'port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnPortNotANumber commentStamp: 'SvenVanCaekenberghe 2/28/2011 15:23' prior: 0!
ZnPortNotANumber is signalled when an URL is parsed containing an illegal port specification.

Part of Zinc HTTP Components.!

!ZnPortNotANumber class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: port
	^ self new
		port: port;
		yourself! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port
	^ port! !

!ZnPortNotANumber methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 15:22'!
port: anObject
	port := anObject! !

Error subclass: #ZnTooManyDictionaryEntries
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnTooManyDictionaryEntries commentStamp: 'SvenVanCaekenberghe 1/2/2012 19:22' prior: 0!
ZnTooManyDictionaryEntries is signalled when too many entries are added to a limited ZnMultiValueDictionary.

Part of Zinc HTTP Components. !

!ZnTooManyDictionaryEntries class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit
	^ limit! !

!ZnTooManyDictionaryEntries methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 19:21'!
limit: anObject
	limit := anObject! !

Error subclass: #ZnUnknownScheme
	instanceVariableNames: 'scheme'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-Resource-Meta-Core'!

!ZnUnknownScheme commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownScheme is signalled when an unknown/unsupported URL scheme is found.

Part of Zinc HTTP Components. !

!ZnUnknownScheme class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/18/2011 13:17'!
scheme: scheme
	^ self new
		scheme: scheme;
		yourself! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme
	^ scheme! !

!ZnUnknownScheme methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/18/2011 13:16'!
scheme: anObject
	scheme := anObject! !
ZnMimeType initialize!
